package aero.sita.lab.thack.ise.tourcms;

import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URIUtils;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import aero.sita.lab.thack.ise.tourcms.model.Response;

/**
 *
 * @author Andy.Malinofsky
 *
 */

public class TourSearch {

   private static final Logger logger = Logger.getLogger(TourSearch.class);
   
   private String scheme = "https";
   private String host = "api.tourcms.com";
   private String path = "/p/tours/search.xml";
   private String marketId = "";
   
   private SignatureGenerator sg = new SignatureGenerator();
   
   /**
    * Call the Tour CMS api a
    * @param activity array of activity srings. Containing what to search for (e.g. fishing, cycling)
    * @param location The location to search 
    * @param country The country to search
    * @param months array of month strings to search for sales.
    * @return XML response from Tour CMS.
    * @throws Exception
    */
   public String getTours(String[] activity, String location, String country, String[] months)throws Exception {
      
      Response rsp = null;
      String body = null;
     
      long time = System.currentTimeMillis();
      DateTime dt = new DateTime(time);
      DateTime dtGmt = dt.withZone(DateTimeZone.UTC);
      DateTimeFormatter fmt = DateTimeFormat.forPattern("EEE, dd MMM yyyy HH:mm:ss zzz");
      String currDate = fmt.print(dtGmt);
      
      List<NameValuePair> qParams = new ArrayList<NameValuePair>();
      //qParams.add(new BasicNameValuePair("lat", "34.052234"));
      //qParams.add(new BasicNameValuePair("long", "-118.243685"));
      if (country != null) {
         qParams.add(new BasicNameValuePair("country", country));   
      }
      if (location != null) {
         qParams.add(new BasicNameValuePair("location", location));   
      }
     
      if (activity != null && activity.length > 0) {
         qParams.add(new BasicNameValuePair("k", activity[0]));
         if (activity.length > 1) {
            qParams.add(new BasicNameValuePair("k2", activity[1]));   
         }
         if (activity.length > 2) {
            qParams.add(new BasicNameValuePair("k3", activity[2]));   
         }
      }
      
      if (months != null && months.length > 0) {
         String saleMonths = "";
         int x = months.length;
         for (int y = 0; y < x; y++) {
            saleMonths += months[y];
            if (y < x-1) {
               saleMonths += ",";
            }
         }
         qParams.add(new BasicNameValuePair("has_sale_month", saleMonths));
      }
      
      URI uri = URIUtils.createURI(scheme, host, -1, path, URLEncodedUtils.format(qParams, "UTF-8"), null);
      HttpGet httpGet = new HttpGet(uri);
      httpGet.setHeader("x-tourcms-date", currDate);
      String sUri = httpGet.getURI().toString();
      
      sUri = sUri.substring(23);
      logger.info(sUri);
      String authString = sg.encode(sUri, "GET", "0", time);
      logger.info(authString);
      httpGet.setHeader("Authorization", "TourCMS 0:"+marketId+":"+authString);

      HttpClient httpclient = new DefaultHttpClient();
      HttpResponse response = null;
      try {
         response = httpclient.execute(httpGet);
      }
      catch (IOException ioe) {
         //logger.error("error sending request: " + ioe.getMessage());
         throw new Exception(ioe.getMessage(), ioe);
      }
      HttpEntity entity = response.getEntity();
      if (entity != null) {
         try {
            body = EntityUtils.toString(entity);
            httpGet.abort();
         }
         catch (IOException ioe2) {
//            logger.error("error extracting entity from post response: " + ioe2.getMessage());

         }
      }
      //IMarshaller marshaller = new TourcmsMarshaller();
      //rsp = marshaller.unmarshal(body);

       return body;
   }
   
   /**
    * test only
    * @param args
    */
   public static void main(String[] args) {
      Response rsp = null;
      TourSearch ts = new TourSearch();
      try {
         rsp = ts.getTours(new String[]{"rafting"}, "Orchy", "GB", new String[]{"2", "3"});
      } catch (Exception e) {
         e.printStackTrace();
      }
      System.out.println(new TourcmsMarshaller().marshal(rsp));
   }

}


